/**
 * @file
 * VuoList_VuoColor C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoColor_H
#define VuoList_VuoColor_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoColor VuoList_VuoColor
 * A list of @ref VuoColor elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoColor_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoColor elements.
 */
typedef const struct VuoList_VuoColor_struct { void *l; } * VuoList_VuoColor;
/// @}
#endif

/**
 * Creates a new list of @ref VuoColor elements.
 */
VuoList_VuoColor VuoListCreate_VuoColor(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoColor to quickly initialize a large list.
 */
VuoList_VuoColor VuoListCreateWithCount_VuoColor(const unsigned long count, const VuoColor value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoColor VuoListCopy_VuoColor(const VuoList_VuoColor list);

/**
 * Returns the @ref VuoColor at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoColor VuoListGetValue_VuoColor(const VuoList_VuoColor list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoColor *VuoListGetData_VuoColor(const VuoList_VuoColor list);

/**
 * Changes the @ref VuoColor at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoColor(const VuoList_VuoColor list, const VuoColor value, const unsigned long index);

/**
 * Inserts the `VuoColor` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoColor(const VuoList_VuoColor list, const VuoColor value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoColor(VuoList_VuoColor list, const VuoColor value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoColor(VuoList_VuoColor list, const VuoColor value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoColor(VuoList_VuoColor list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoColor_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoColor(VuoList_VuoColor list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoColor_areEqual(const VuoList_VuoColor a, const VuoList_VuoColor b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoColor_isLessThan(const VuoList_VuoColor a, const VuoList_VuoColor b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoColor(VuoList_VuoColor list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoColor(VuoList_VuoColor list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoColor(VuoList_VuoColor list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoColor(VuoList_VuoColor list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoColor(VuoList_VuoColor list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoColor(VuoList_VuoColor list);

/**
 * Removes the `VuoColor` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoColor(VuoList_VuoColor list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoColor(const VuoList_VuoColor list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoColor VuoList_VuoColor_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoColor_getJson(const VuoList_VuoColor value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoColor_getSummary(const VuoList_VuoColor value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoColor VuoList_VuoColor_makeFromString(const char *str);
char * VuoList_VuoColor_getString(const VuoList_VuoColor value);
/// @}

/**
 * @}
 */

#endif
