/**
 * @file
 * VuoList_VuoImageWrapMode C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoImageWrapMode_H
#define VuoList_VuoImageWrapMode_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoImageWrapMode VuoList_VuoImageWrapMode
 * A list of @ref VuoImageWrapMode elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoImageWrapMode_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoImageWrapMode elements.
 */
typedef const struct VuoList_VuoImageWrapMode_struct { void *l; } * VuoList_VuoImageWrapMode;
/// @}
#endif

/**
 * Creates a new list of @ref VuoImageWrapMode elements.
 */
VuoList_VuoImageWrapMode VuoListCreate_VuoImageWrapMode(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoImageWrapMode to quickly initialize a large list.
 */
VuoList_VuoImageWrapMode VuoListCreateWithCount_VuoImageWrapMode(const unsigned long count, const VuoImageWrapMode value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoImageWrapMode VuoListCopy_VuoImageWrapMode(const VuoList_VuoImageWrapMode list);

/**
 * Returns the @ref VuoImageWrapMode at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoImageWrapMode VuoListGetValue_VuoImageWrapMode(const VuoList_VuoImageWrapMode list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoImageWrapMode *VuoListGetData_VuoImageWrapMode(const VuoList_VuoImageWrapMode list);

/**
 * Changes the @ref VuoImageWrapMode at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoImageWrapMode(const VuoList_VuoImageWrapMode list, const VuoImageWrapMode value, const unsigned long index);

/**
 * Inserts the `VuoImageWrapMode` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoImageWrapMode(const VuoList_VuoImageWrapMode list, const VuoImageWrapMode value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoImageWrapMode(VuoList_VuoImageWrapMode list, const VuoImageWrapMode value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoImageWrapMode(VuoList_VuoImageWrapMode list, const VuoImageWrapMode value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoImageWrapMode(VuoList_VuoImageWrapMode list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoImageWrapMode_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoImageWrapMode(VuoList_VuoImageWrapMode list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoImageWrapMode_areEqual(const VuoList_VuoImageWrapMode a, const VuoList_VuoImageWrapMode b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoImageWrapMode_isLessThan(const VuoList_VuoImageWrapMode a, const VuoList_VuoImageWrapMode b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoImageWrapMode(VuoList_VuoImageWrapMode list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoImageWrapMode(VuoList_VuoImageWrapMode list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoImageWrapMode(VuoList_VuoImageWrapMode list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoImageWrapMode(VuoList_VuoImageWrapMode list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoImageWrapMode(VuoList_VuoImageWrapMode list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoImageWrapMode(VuoList_VuoImageWrapMode list);

/**
 * Removes the `VuoImageWrapMode` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoImageWrapMode(VuoList_VuoImageWrapMode list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoImageWrapMode(const VuoList_VuoImageWrapMode list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoImageWrapMode VuoList_VuoImageWrapMode_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoImageWrapMode_getJson(const VuoList_VuoImageWrapMode value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoImageWrapMode_getSummary(const VuoList_VuoImageWrapMode value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoImageWrapMode VuoList_VuoImageWrapMode_makeFromString(const char *str);
char * VuoList_VuoImageWrapMode_getString(const VuoList_VuoImageWrapMode value);
/// @}

/**
 * @}
 */

#endif
