/**
 * @file
 * VuoList_VuoSerialDevice C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoSerialDevice_H
#define VuoList_VuoSerialDevice_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoSerialDevice VuoList_VuoSerialDevice
 * A list of @ref VuoSerialDevice elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoSerialDevice_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoSerialDevice elements.
 */
typedef const struct VuoList_VuoSerialDevice_struct { void *l; } * VuoList_VuoSerialDevice;
/// @}
#endif

/**
 * Creates a new list of @ref VuoSerialDevice elements.
 */
VuoList_VuoSerialDevice VuoListCreate_VuoSerialDevice(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoSerialDevice to quickly initialize a large list.
 */
VuoList_VuoSerialDevice VuoListCreateWithCount_VuoSerialDevice(const unsigned long count, const VuoSerialDevice value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoSerialDevice VuoListCopy_VuoSerialDevice(const VuoList_VuoSerialDevice list);

/**
 * Returns the @ref VuoSerialDevice at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoSerialDevice VuoListGetValue_VuoSerialDevice(const VuoList_VuoSerialDevice list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoSerialDevice *VuoListGetData_VuoSerialDevice(const VuoList_VuoSerialDevice list);

/**
 * Changes the @ref VuoSerialDevice at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoSerialDevice(const VuoList_VuoSerialDevice list, const VuoSerialDevice value, const unsigned long index);

/**
 * Inserts the `VuoSerialDevice` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoSerialDevice(const VuoList_VuoSerialDevice list, const VuoSerialDevice value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoSerialDevice(VuoList_VuoSerialDevice list, const VuoSerialDevice value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoSerialDevice(VuoList_VuoSerialDevice list, const VuoSerialDevice value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoSerialDevice(VuoList_VuoSerialDevice list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoSerialDevice_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoSerialDevice(VuoList_VuoSerialDevice list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoSerialDevice_areEqual(const VuoList_VuoSerialDevice a, const VuoList_VuoSerialDevice b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoSerialDevice_isLessThan(const VuoList_VuoSerialDevice a, const VuoList_VuoSerialDevice b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoSerialDevice(VuoList_VuoSerialDevice list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoSerialDevice(VuoList_VuoSerialDevice list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoSerialDevice(VuoList_VuoSerialDevice list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoSerialDevice(VuoList_VuoSerialDevice list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoSerialDevice(VuoList_VuoSerialDevice list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoSerialDevice(VuoList_VuoSerialDevice list);

/**
 * Removes the `VuoSerialDevice` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoSerialDevice(VuoList_VuoSerialDevice list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoSerialDevice(const VuoList_VuoSerialDevice list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoSerialDevice VuoList_VuoSerialDevice_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoSerialDevice_getJson(const VuoList_VuoSerialDevice value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoSerialDevice_getSummary(const VuoList_VuoSerialDevice value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoSerialDevice VuoList_VuoSerialDevice_makeFromString(const char *str);
char * VuoList_VuoSerialDevice_getString(const VuoList_VuoSerialDevice value);
/// @}

/**
 * @}
 */

#endif
